<?php
/********************************************************************
 * (c) Copyright Sysload Software 2005-2013
 * FILE       : sppadmtask.inc.php $
 * LAST USER  : $LastChangedBy::                        $
 * MODTIME    : $LastChangedDate::                      $
 * REVISION   : $LastChangedRevision::                  $
 *
 * DESCRIPTION:  
 * HISTORY    :
 * 18/01/10 PBN> Created. 
 *******************************************************************/
require_once (LIB_DIR."classes/sld_user.class.php");
require_once (COMMON_DIR."sldf_senddata.php");
require_once (COMMON_DIR."sldf_admin_log.php");
require_once (COMMON_DIR."sldf_admin_tasks.php");
require_once (COMMON_DIR."sldf_admin_tasks_external_data.php");
require_once (COMMON_DIR."sldf_time.php");
require_once (COMMON_DIR."sldf_folders.php");
require_once (COMMON_DIR."sldf_ondemandreport.php");
require_once (COMMON_DIR."sld_parameters.php");
    
class sppadmtask
{
  
//===================================================================
// get_tasklog_list()
//===================================================================
  static function get_tasklog_list($arStrLang)
  {
    global $pconnection;
  
    // vrif autorisation seul Admins peut faire cette opration
    if (!sld_user::is_role(ROLE_ADMINS))
    {
      $strMsg = $arStrLang["SLD_LNG_ADM_MSG_NO_AUTHORISATION"];
      pfn_response(0,$strMsg);  
    }
    $arTaskLog = array();
    $arRet     = array();

    $nTotalCount     = 0;
    $nOffset         = isset($_POST["start"]) ? $_POST["start"] : 0;    
    $nElementsByPage = isset($_POST["limit"]) ? $_POST["limit"] : -1;
    
    $arTaskLog["data"] = sld_read_adminlog_list ($nTotalCount /* out*/,  
                                                 $nOffset /* out*/, 
                                                 $pconnection,
                                                 "", /* no filter */
                                                 $nElementsByPage );
    $arTaskLog["totalSize"]  = $nTotalCount;
    $arTaskLog["nbelements"] = $nElementsByPage;
    
    $bSendAdditional = isset($_POST['additional_info']);
    
    foreach ($arTaskLog["data"] as $index => &$arLog)
    {
      /* additional data */
      if ($bSendAdditional)
      {
        $arTaskInfo = array( SLD_ADMINTASK_TYPE      => $arLog[SLD_ADMINLOG_OBJTYPE],
                             SLD_ADMINTASK_PARAMLIST => $arLog[SLD_ADMINLOG_OBJDETAIL],
                             SLD_ADMINTASK_STATUS    => $arLog[SLD_ADMINLOG_OBJSTATUS],
                             SLD_ADMINTASK_STARTTIME => $arLog[SLD_ADMINLOG_OBJSTARTDATE]);
        // Long Label            
        $strTypeLabel  = sld_get_admintask_type_label($arTaskInfo, $arStrLang);
        if (!empty($arLog[SLD_ADMINLOG_OBJNAME]))
          $strTypeLabel .= " - ".$arLog[SLD_ADMINLOG_OBJNAME];
        
        // Detail
        $strTaskStartDate = $arLog[SLD_ADMINLOG_OBJSTARTDATE];
        $strTaskDetail = sld_get_admintask_detail_label($arTaskInfo, $arStrLang);
        // Status
        $strStatus = sld_get_admintask_lastresult_label($arTaskInfo, $arStrLang);
        // Failed Agent Number (collecte)
        $nCount = sld_externaldata_failed_agent_number(NULL, $arLog[SLD_ADMINLOG_ID], $pconnection);
        
        $arLog['sld_additional_failed_agents'] = $nCount;
        $arLog['sld_additional_label']         = $strTypeLabel;
        $arLog['sld_additional_detailhtml']    = $strTaskDetail;
        $arLog['sld_additional_resulthtml']    = $strStatus;
        
      }
    }
    
    return $arTaskLog;
    
  } /* get_tasklog_list */
  
//===================================================================
// clear_tasklog
//===================================================================
  static function clear_tasklog($arStrLang)
  {
    global $pconnection;
    
    $strMsg = $arStrLang["SLD_LNG_UNEXPECTED_ERROR"];
    $nRet   = 0;

    // vrif autorisation seul Admins peut faire cette opration
    if (!sld_user::is_role(ROLE_ADMINS))
    {
      $strMsg = $arStrLang["SLD_LNG_ADM_MSG_NO_AUTHORISATION"];
      pfn_response(0, $strMsg);
    }
  
    if (sld_clear_adminlog ($pconnection, 0))
    {
      $strMsg = "";
      $nRet   = 1; // success
    }
    
    pfn_response($nRet, $strMsg);

  } /* clear_tasklog */
  
//===================================================================
// run_task_from_log
//===================================================================
  static function run_task_from_log($arStrLang)
  {
    global $pconnection;
    
    $tasklogid  = $_POST["tasklog_id"];
    $bIsPartial = $_POST["partial"] == 1 ? true : false;
    $strMsg = $arStrLang["SLD_LNG_UNEXPECTED_ERROR"];
    $nRet   = 0;

    // vrif autorisation seul Admins peut faire cette opration
    if (!sld_user::is_role(ROLE_ADMINS))
    {
      $strMsg = $arStrLang["SLD_LNG_ADM_MSG_NO_AUTHORISATION"];
      pfn_response(0, $strMsg);
    }

    if (sld_run_task_from_log ($pconnection, $tasklogid, $bIsPartial))
    {
      $strMsg = $arStrLang["SLD_LNG_ADM_MSG_TASK_START"];
      $nRet = 1; // success
    }
    
    pfn_response($nRet, $strMsg);

  } /* run_task_from_log */
  
//===================================================================
// export_unreachable_agents()
//===================================================================
  static function export_unreachable_agents($arStrLang)
  {
    global $pconnection;
    global $g_TraceLevel;
    
    $logid = isset($_GET['logid']) ? intval($_GET['logid']) : 0;
    
    $sql =  "SELECT distinct sld_mgtserver_logical_name as MgtServer,"
                    ."sld_device_hostname as HostName,"
                    ."sld_device_instance as Instance,"
                    ."sld_devicetype_shortdesc as AgentType,"
                    ."sld_device_key as AgentKey"
            ." FROM ".SLD_TABLE_ADMINTASKS_EXTERNAL_DATA
            ." INNER JOIN sld_device ON sld_device.sld_device_id=CAST(sld_external_data_val AS INTEGER)"
            ." INNER JOIN sld_mgtserver ON sld_mgtserver.sld_mgtserver_id=sld_device.sld_mgtserver_id"
            ." INNER JOIN sld_devicetypedesc ON sld_devicetypedesc.sld_devicetype_key=sld_device.sld_devicetype_key"
            ." INNER JOIN sld_primarylanguage ON sld_primarylanguage.sld_primarylanguage_id=sld_devicetypedesc.sld_primarylanguage_id"
            ." WHERE sld_external_data_key = '".SLD_EXTERNAL_DATA_FAILED_AGENTS."'"
            ." AND sld_log_id = $logid"
            ." AND sld_primarylanguage_shortlabel = '".$arStrLang["SLD_LNG_CODE"]."'"
            ." ORDER BY 1,2,3,4";
    if ($g_TraceLevel > 0)
      error_log ("export_unreachable_agents : ".$sql);
            
    $rset = $pconnection->Execute($sql);
    
    sld_send_data_csv($rset, ".", TRUE /* header */, FALSE /* callback*/, "devices.csv");
  } /* export_unreachable_agents */
  
//===================================================================
// get_tasklist()
//===================================================================
  static function get_tasklist($arStrLang)
  {
    global $pconnection;
    
    $arRet = array();
    $arTS  = array();
    
    $arTaskList = sld_read_admintask_list ( $pconnection, true, "");
    
    // tri sur timestamp dcroissant
    foreach ($arTaskList as $key => $row)
    {
      if (($row[SLD_ADMINTASK_STATUS] == SLD_ADMTASKSTATUS_PLANED && $row[SLD_ADMINTASK_SCHEDULING]==SCHEDULING_ONDEMAND) || 
            $row[SLD_ADMINTASK_STATUS] == SLD_ADMTASKSTATUS_PROCESSING)
        $bHasRunningTasks = true;

      $arTS[$key]  = $row[SLD_ADMINTASK_LASTCOMPLETED];
    }
      
    array_multisort($arTS, SORT_DESC, SORT_NUMERIC, $arTaskList);

    foreach ($arTaskList as $index => $arTaskInfo)
    {
      // skip Internal tasks + Terminated tasks
      if (!sppadmtask::is_task_visible($arTaskInfo[SLD_ADMINTASK_TYPE]) ||
          ($arTaskInfo[SLD_ADMINTASK_SCHEDULING]==SCHEDULING_ONDEMAND && $arTaskInfo[SLD_ADMINTASK_STATUS]>SLD_ADMTASKSTATUS_PROCESSING))
        continue;
      
      // Next execution
      $tsNext = 0;
      if ($arTaskInfo[SLD_ADMINTASK_SCHEDULING] != SCHEDULING_ONDEMAND)
      {
        $bShouldRun       = FALSE;
        sld_get_admintask_nextexecution($tsNext /* out */, $bShouldRun /* out */, $arTaskInfo);
      }
      // Type Label
      $strTypeLabel  = sld_get_admintask_type_label($arTaskInfo, $arStrLang);
      // Detail HTML
      $strTaskDetail = sld_get_admintask_detail_label($arTaskInfo, $arStrLang);
      // Result HTML
      $strResult     = sld_get_admintask_lastresult_label($arTaskInfo, $arStrLang);
      // Scheduling
      $strScheduling = sld_get_admintask_scheduling_label($arTaskInfo, $arStrLang);
      // Last result
      $strLastResult = sld_get_admintask_lastresult_label($arTaskInfo, $arStrLang);    
      // Status
      $strStatus     = sld_get_admintask_status_label($arTaskInfo, $arStrLang, false /* icon */);

      $arTaskInfo['sld_additional_nextts']      = empty($arTaskInfo[SLD_ADMINTASK_LASTCOMPLETED]) ? 0 : $tsNext;
      $arTaskInfo['sld_additional_typelabel']   = $strTypeLabel;
      $arTaskInfo['sld_additional_detailhtml']  = $strTaskDetail;
      $arTaskInfo['sld_additional_resulthtml']  = $strLastResult;
      $arTaskInfo['sld_additional_statushtml']  = $strStatus;
      $arTaskInfo['sld_additional_schedlabel']  = $strScheduling;
                
      $arRet[] = $arTaskInfo;
    } 
               
    return $arRet;    
    
  } /* get_tasklist */
  
//===================================================================
// is_task_visible()
//===================================================================
  static function is_task_visible($nType)
  {
    return ($nType != SLD_ADMINTASK_REPORTTIMEOUT &&
            $nType != SLD_ADMINTASK_LOGPURGE); 
  } /* is_task_visible */
  
//===================================================================
// get_task()
//===================================================================
  static function get_task($arStrLang)
  {
    global $pconnection;
    $arRet    = array();
    $arParams = array();
    $arData   = array();
    
    $task_id          = $_POST["task_id"];
    
    if (!empty($task_id))
    {
      $arAdminTask = sld_read_admintask ($task_id , $pconnection);
      if (!empty($arAdminTask))
      {
        $arParams      = $arAdminTask[SLD_ADMINTASK_PARAMLIST];     
        $arData = array ('admintask_id'             => $arAdminTask[SLD_ADMINTASK_ID],
                         'admintask_type'           => $arAdminTask[SLD_ADMINTASK_TYPE],
                         'scheduling'               => $arAdminTask[SLD_ADMINTASK_SCHEDULING],
                         'scheduling_day'           => $arAdminTask[SLD_ADMINTASK_SCHEDULINGDAY],
                         'scheduling_hour'          => $arAdminTask[SLD_ADMINTASK_SCHEDULINGHOUR],
                         'scheduling_minute'        => $arAdminTask[SLD_ADMINTASK_SCHEDULINGMINUTE],
                         'admintask_lastcompleted'  => $arAdminTask[SLD_ADMINTASK_LASTCOMPLETED],
                         'admintask_name'           => $arAdminTask[SLD_ADMINTASK_NAME],
                         'admintask_disabled'       => ($arAdminTask[SLD_ADMINTASK_DISABLED]==1) ? 1 : 0,
                         'admintask_params'         => $arParams      /* array */);
      }
    }
       
    if (!empty($arData))
      $arRet[] = $arData;
      
    return $arRet;
  
  } /* get_task */
  
//===================================================================
// run_task()
//===================================================================
  static function run_task ($arStrLang)
  {
    global $pconnection;
    
    $strMsg      = $arStrLang["SLD_LNG_UNEXPECTED_ERROR"];    
    $nTaskId     = $_POST["task_id"];
    $arAdminTask = sld_read_admintask ($nTaskId, $pconnection);

    if (!empty($arAdminTask))
    {
      $bErrors = false;
      // une tche est elle dja en cours
      if (!sppadmtask::check_can_add($arAdminTask[SLD_ADMINTASK_TYPE]))
      {
        $strMsg = $arStrLang["SLD_LNG_ADM_MSG_TASK_ALTREADY_STARTED"];  
        $bErrors = true;
      }
      // suppression des taches  execution immdiate termine de ce type
      $arStatus = array(SLD_ADMTASKSTATUS_COMPLETED, 
                        SLD_ADMTASKSTATUS_FAILED, 
                        SLD_ADMTASKSTATUS_WARNING);
                                  
      if (!$bErrors && !delete_task_from_type (intval($arAdminTask[SLD_ADMINTASK_TYPE]), $pconnection, SCHEDULING_ONDEMAND, $arStatus))
      {
        $strMsg = $arStrLang["SLD_LNG_ADM_MSG_ERROR_OCCURED"];  
        $bErrors = true;            
      }          
      
      if (!$bErrors)
      {
        $arAdminTask[SLD_ADMINTASK_ID]            = NEWADMINTASK_ID;
        $arAdminTask[SLD_ADMINTASK_SCHEDULING]    = SCHEDULING_ONDEMAND;
        $arAdminTask[SLD_ADMINTASK_LASTCOMPLETED] = 0;
        $arAdminTask[SLD_ADMINTASK_STATUS]        = SLD_ADMTASKSTATUS_PLANED;
        $arAdminTask[SLD_ADMINTASK_DISABLED]      = 0;
        $nTaskId = "";
        if (!sld_save_admintask ($nTaskId /* out */, $arAdminTask, $pconnection, true))
        {
          $strMsg = $arStrLang["SLD_LNG_ADM_MSG_SAVE_FAILED"];          
        }
        else
        {
          // succes
          $strMsg = $arStrLang["SLD_LNG_ADM_MSG_TASK_START"];
          pfn_response(1, $strMsg);
        }
      }        
    }
    else
    {
      // error
      error_log(basename(__FILE__).' '.'['.__LINE__.']: Could not read admin task :'.$nTaskId);
      $strMsg = $arStrLang["SLD_LNG_ADM_MSG_ERROR_OCCURED"];
    }

    pfn_response(0, $strMsg);
 
  } /* run_task */
    
//===================================================================
// cancel_task ()
//===================================================================
  static function cancel_task ($arStrLang)
  {
    global $pconnection;
    $nRet = 0;
    
    // vrif autorisation seul Admins peut faire cette opration
    if (!sld_user::is_role(ROLE_ADMINS))
    {
      $strMsg = $arStrLang["SLD_LNG_ADM_MSG_NO_AUTHORISATION"];
      pfn_response(0, $strMsg);
    }
    
    $strMsg      = $arStrLang["SLD_LNG_UNEXPECTED_ERROR"];    
    $nTaskId     = $_POST["task_id"];
    
    $arAdminTask = sld_read_admintask ($nTaskId, $pconnection);
    
    if (!empty($arAdminTask))
    {
      $bErrors = false;
      if ($arAdminTask[SLD_ADMINTASK_STATUS] == SLD_ADMTASKSTATUS_TERMINATING)
      {
        // efface le status dans la base
        sld_update_admintask_status ($arAdminTask, SLD_ADMTASKSTATUS_WARNING, $arStrLang["SLD_LNG_ADM_TASK_CANCELLED"], $pconnection);
        $strMsg = $arStrLang["SLD_LNG_ADM_TASK_CANCELLED"];
        $nRet   = 1;
      }
      else
      {
        if ($arAdminTask[SLD_ADMINTASK_TYPE] == SLD_ADMINTASK_SLDPPCOLL)
        {
          // cancel data collection task, special case

          // behavior prior to 6.13 :
          //sppadmtask::cancel_datacoll_soft();

          // new behavior starting 6.13 :
          sppadmtask::cancel_datacoll_hard();
        }
  
        $arAdminTask[SLD_ADMINTASK_STARTTIME] = time();
        if ( sld_update_admintask_status ($arAdminTask, SLD_ADMTASKSTATUS_TERMINATING, "", $pconnection, false /* no log */ ))
        {
          // success
          $strMsg = $arStrLang["SLD_LNG_ADM_TASK_CANCELLED"];
          $nRet   = 1;
        }
        else
        {
          $strMsg = $arStrLang["SLD_LNG_ADM_MSG_ERROR_DELETETASK"];
        }
      }
    }
    
    pfn_response($nRet, $strMsg);
  } /* cancel_task */
  
//===================================================================
// cancel_datacoll_soft ()
// create an halt file so data collection knows it should shut down
//===================================================================
  static function cancel_datacoll_soft()
  { 
    $bRet = false;
    $collDir = sld_Get_sldppcoll_path();
    if (empty($collDir))
    {
      // on suppose que sldppcoll est dans "../../../sldppcoll/";
      $collDir = @realpath(SLDPPCOLL_RELPATH);
      $collDir.= "/".SLDPPCOLL_FOLDER;
    }  
    $collDir  = @realpath($collDir);
    if (file_put_contents ( $collDir."\\".SLDPPCOLL_HALTFILE , "") === FALSE)
      error_log(basename(__FILE__).' '.'['.__LINE__.']: Failed to create sldppcoll HALT file' );   
    else
      $bRet = true;
    
    return $bRet;
  } /* cancel_datacoll_soft */
  
//===================================================================
// cancel_datacoll_hard ()
// try to kill the process in memory
//===================================================================
  static function cancel_datacoll_hard()
  {
    $bRet = false;
    $image_name = str_replace(".exe", "*", SLDPPCOLL_EXENAME); // sldppcoll.exe => sldppcoll*
    $command = "taskkill /F /IM ".$image_name." 2>&1";
    error_log(SLD_LOG_TASKINFO.'Cancelling data collection, executing command:'.$command);      
    $arOutput = array();
    @exec ( $command, $arOutput, $return_var );
    
    // Ensure the lock file is removed
    sppadmtask::delete_lock_file();
    
    if ($return_var == 0)
    {
      error_log(SLD_LOG_TASKINFO.'Process '.SLDPPCOLL_EXENAME.' successfully killed.');  
      $bRet = true;
    }
    else
    {
      $output = implode(",", $arOutput);
      error_log(basename(__FILE__).' '.'['.__LINE__.']: Failed to kill process '.SLDPPCOLL_EXENAME.' :'.$output );   
    }
    
    return $bRet;
  } /* cancel_datacoll_hard */
  
//===================================================================
// delete_lock_file ()
//===================================================================
  static function delete_lock_file()
  {
    $image_name_lck = str_replace(".exe", ".lck", SLDPPCOLL_EXENAME);

    $lockFilePath = sld_Get_sldppcoll_path();
    if (empty($lockFilePath))
    {
      // Assume sldppcoll is in "../../../sldppcoll/";
      $lockFilePath = @realpath(SLDPPCOLL_RELPATH);
      $lockFilePath.= "/".SLDPPCOLL_FOLDER;
    }
    $lockFilePath  = @realpath($lockFilePath);    
    $lockFilePath.= "/".$image_name_lck; 
    
    if (@file_exists($lockFilePath)) 
    {
      error_log(SLD_LOG_TASKINFO.'Deleting lock file: '.$lockFilePath);  
      if (@unlink($lockFilePath) === FALSE)
      {
        error_log(basename(__FILE__).' '.'['.__LINE__.']: Error, could not delete lock file: '.$lockFilePath );   
      }
      else
      {
        error_log(SLD_LOG_TASKINFO.'Lock file successfully deleted: '.$lockFilePath);  
      }
    }
    else
      error_log(SLD_LOG_TASKINFO.'Lock file not found: '.$lockFilePath);
    
  } /* delete_lock_file */
  
//===================================================================
// delete_task ()
//===================================================================
  static function delete_task ($arStrLang)
  {
    global $pconnection;
    $nRet = 0;
    
    $strMsg  = $arStrLang["SLD_LNG_UNEXPECTED_ERROR"];    
    $nTaskId = $_POST["task_id"];
    
    // vrif autorisation seul Admins peut faire cette opration
    if (!sld_user::is_role(ROLE_ADMINS))
    {
      $strMsg = $arStrLang["SLD_LNG_ADM_MSG_NO_AUTHORISATION"];
      pfn_response(0, $strMsg);
    }
    
    if (sppadmtask::check_can_delete($strMsg,$nTaskId, $arStrLang ) && 
        delete_task_from_id ($nTaskId, $pconnection))
    {
      $strMsg = "";
      $nRet   = 1; // success
    }
    else
      $strMsg = $arStrLang["SLD_LNG_ADM_MSG_ERROR_DELETETASK"];
    
    pfn_response($nRet, $strMsg);    
  }
  
//===================================================================
// check_can_delete ()
//===================================================================    
  static function check_can_delete(&$o_strMessages, $nTaskId, $arStrLang)
  {
    global $pconnection;
    $bRet = false;
    $arAdminTasks = sld_read_admintask ($nTaskId, $pconnection, false);

    if ($arAdminTasks[SLD_ADMINTASK_STATUS] != SLD_ADMTASKSTATUS_PROCESSING)
    {
      $bRet=true;
    }
    else
    {
      $strErrMsg = $arStrLang["SLD_LNG_ADM_MSG_TASK_DELETE_RUNNING"]." ID:".$nTaskId;
      $o_aMessages[] = array("status"=>"error", "message"=> $strErrMsg);
    }
    return $bRet;
  } /* check_can_delete */
    
//===================================================================
// check_can_add ()
//===================================================================    
  static function check_can_add($nTaskType)
  {
    global $pconnection;
    $bRet = false;
    $arAdminTasks = sld_read_admintask_list ( $pconnection, false, $nTaskType);
    if ($arAdminTasks!==0)
    {
      foreach ($arAdminTasks as $ID => $taskInfo)
      {
        if ($taskInfo[SLD_ADMINTASK_SCHEDULING] == SCHEDULING_ONDEMAND &&
            $taskInfo[SLD_ADMINTASK_STATUS] < SLD_ADMTASKSTATUS_COMPLETED)
        {
          return false;
        }
      }
      $bRet = true;
    }
    return $bRet;
  } /* check_can_add */
    
}